// -*- mode: c++ -*-
// OpenMP+CUDA skeleton for the problem of successive sorts and squares in a matrix
// Spanish Parallel Programming Contest 2012

#include <stdlib.h>
#include <stdio.h>
#include <sys/time.h>
#include <omp.h>
#include <signal.h>
#include <unistd.h>

#include <cutil_inline.h>

//////////////////////////////////////////////////////

void initialize(int *m,int t,int maximum)
{
  int i;

// Integer numbers are generated randomly
  for(i=0;i<t*t;i++)
  {  
    m[i] =(int) (((1.*rand())/RAND_MAX)*(2*maximum+1))-maximum;
  }
}

void escribir(int *m, int t){
  int i, j;

  for (i = 0; i < t; i++)
  {
    for (j = 0; j < t; j++)
    {  
      printf("%d ",m[i*t+j]);
    }
    printf("\n");
  }
  printf("\n");
}

void escribirresult(int n,int *m, int stride){
  int i;

  for (i = 0; i < n*n; i+=stride)
  {
    printf("%d ",m[i]);
  }
  printf("\n");
}

/*
c
c     mseconds - returns elapsed milliseconds since Jan 1st, 1970.
c
*/
long long mseconds(void){
  struct timeval t;
  gettimeofday(&t, NULL);
  return t.tv_sec*1000 + t.tv_usec/1000;
}

static void alarm_handler(int sig) {
  fprintf(stderr, "Time Limit Exceeded\n");
  abort();
}

void sort_square(int steps,int t,int *a,int node,int np);

int main(int argc,char *argv[]) {
  int node,i,N,steps,nproblems,stride,seed,maximum;
  long long ti,tf,tt=0;
  int *a;
  int np=1;

  FILE *stats_file = fopen("stats", "w");

  struct sigaction sact;
  sigemptyset(&sact.sa_mask);
  sact.sa_flags = 0;
  sact.sa_handler = alarm_handler;
  sigaction(SIGALRM, &sact, NULL);
  alarm(20);  /* time limit */
 

// The number of test cases is read
// and sent to all the processes
  scanf("%d",&nproblems);

  for(i=0;i<nproblems;i++)
  {
      scanf("%d",&steps);		// The first argument is the number of sorts-squares
      scanf("%d",&N);                   // The second argument is the matrix size
      scanf("%d",&seed);             // the third is the seed for random generation
      scanf("%d",&maximum);		// the maximum value of the ingeter; which will be generated between -maximum+1 and maximum-1
      scanf("%d",&stride);              // and the stride used to writte the solution

// Space for the matrix
      a = (int *) malloc(sizeof(int)*N*N);

      srand(seed);
      initialize(a,N,maximum);

#ifdef DEBUG
// While debugging the matrix and the numbers are written
      escribir(a,N);
#endif



    ti=mseconds(); 
    sort_square(steps,N,a,node,np);
    tf=mseconds(); 




// The time of the first problem is not considered
      if(i!=0) tt+=tf-ti;
#ifdef TIME
// to escribir the execution time of each running
      fprintf(stats_file, "%Ld\n", tf-ti);
#endif
// The results of each problem are written,
// even for the first problem
      escribirresult(N,a,stride);
      free(a);

  }//for nproblems


    fprintf(stats_file, "%Ld\n", tt);
    fclose(stats_file);

    return 0;
}
